@@ -42,7 +42,7 @@ pub fn generate(ast: &[parser::Token]) -> String {
                 } else {
                     temp
                 };
-                decls += &temp;
+                decls += temp;
                 decls += "\n";
             }
             Class => {
@@ -86,12 +86,20 @@ pub fn generate(ast: &[parser::Token]) -> String {
                 let xywh = props.iter().position(|x| x == "xywh");
                 let label = props.iter().position(|x| x == "label");
                 let typ = props.iter().position(|x| x == "type");
-                let is_parent = match t.as_str() {
-                    "Window" | "Group" | "Pack" | "Tabs" | "Scroll" | "Table" | "Tile"
-                    | "Wizard" => true,
-                    "MenuBar" | "MenuButton" | "Choice" | "InputChoice" => true,
-                    _ => false,
-                };
+                let is_parent = matches!(
+                    t.as_str(),
+                    "Window"
+                        | "Group"
+                        | "Pack"
+                        | "Tabs"
+                        | "Scroll"
+                        | "Table"
+                        | "Tile"
+                        | "Wizard"
+                        | "MenuBar"
+                        | "MenuButton"
+                        | "Choice"
+                );
                 if !is_parent {
                     if t != "MenuItem" && t != "Submenu" {
                         if let Some(xywh) = xywh {
@@ -177,12 +185,24 @@ pub fn generate(ast: &[parser::Token]) -> String {
                 for i in 0..props.len() {
                     match props[i].as_str() {
                         "visible" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!("\t{}.show();\n", &elem.ident,);
+                                }
+                            } else {
                                 imp += &format!("\t{}.show();\n", &elem.ident,);
                             }
                         }
                         "color" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!(
+                                        "\t{}.set_color(Color::by_index({}));\n",
+                                        &elem.ident,
+                                        utils::unbracket(&props[i + 1])
+                                    );
+                                }
+                            } else {
                                 imp += &format!(
                                     "\t{}.set_color(Color::by_index({}));\n",
                                     &elem.ident,
@@ -191,7 +211,15 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "selection_color" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!(
+                                        "\t{}.set_selection_color(Color::by_index({}));\n",
+                                        &elem.ident,
+                                        utils::global_to_pascal(utils::unbracket(&props[i + 1]))
+                                    );
+                                }
+                            } else {
                                 imp += &format!(
                                     "\t{}.set_selection_color(Color::by_index({}));\n",
                                     &elem.ident,
@@ -200,7 +228,15 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "labelsize" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!(
+                                        "\t{}.set_label_size({});\n",
+                                        &elem.ident,
+                                        utils::unbracket(&props[i + 1])
+                                    );
+                                }
+                            } else {
                                 imp += &format!(
                                     "\t{}.set_label_size({});\n",
                                     &elem.ident,
@@ -209,7 +245,15 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "textsize" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!(
+                                        "\t{}.set_text_size({});\n",
+                                        &elem.ident,
+                                        utils::unbracket(&props[i + 1])
+                                    );
+                                }
+                            } else {
                                 imp += &format!(
                                     "\t{}.set_text_size({});\n",
                                     &elem.ident,
@@ -218,7 +262,17 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "labeltype" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    let temp =
+                                        utils::global_to_pascal(utils::unbracket(&props[i + 1]));
+                                    let temp = if temp == "No" { "None" } else { temp.as_str() };
+                                    imp += &format!(
+                                        "\t{}.set_label_type(LabelType::{});\n",
+                                        &elem.ident, temp,
+                                    );
+                                }
+                            } else {
                                 let temp = utils::global_to_pascal(utils::unbracket(&props[i + 1]));
                                 let temp = if temp == "No" { "None" } else { temp.as_str() };
                                 imp += &format!(
@@ -228,7 +282,15 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "labelcolor" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!(
+                                        "\t{}.set_label_color(Color::by_index({}));\n",
+                                        &elem.ident,
+                                        utils::unbracket(&props[i + 1])
+                                    );
+                                }
+                            } else {
                                 imp += &format!(
                                     "\t{}.set_label_color(Color::by_index({}));\n",
                                     &elem.ident,
@@ -237,7 +299,15 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "labelfont" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!(
+                                        "\t{}.set_label_font(Font::by_index({}));\n",
+                                        &elem.ident,
+                                        utils::unbracket(&props[i + 1])
+                                    );
+                                }
+                            } else {
                                 imp += &format!(
                                     "\t{}.set_label_font(Font::by_index({}));\n",
                                     &elem.ident,
@@ -246,7 +316,15 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "textfont" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!(
+                                        "\t{}.set_text_font(Font::by_index({}));\n",
+                                        &elem.ident,
+                                        utils::unbracket(&props[i + 1])
+                                    );
+                                }
+                            } else {
                                 imp += &format!(
                                     "\t{}.set_text_font(Font::by_index({}));\n",
                                     &elem.ident,
@@ -255,7 +333,23 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "box" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    let temp =
+                                        utils::global_to_pascal(utils::unbracket(&props[i + 1]));
+                                    let temp = match temp.as_str() {
+                                        "OflatBox" => "OFlatBox",
+                                        "OshadowBox" => "OShadowBox",
+                                        "RflatBox" => "RFlatBox",
+                                        "RshadowBox" => "RShadowBox",
+                                        _ => temp.as_str(),
+                                    };
+                                    imp += &format!(
+                                        "\t{}.set_frame(FrameType::{});\n",
+                                        &elem.ident, temp,
+                                    );
+                                }
+                            } else {
                                 let temp = utils::global_to_pascal(utils::unbracket(&props[i + 1]));
                                 let temp = match temp.as_str() {
                                     "OflatBox" => "OFlatFrame",
@@ -271,16 +365,47 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "down_box" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    let temp =
+                                        utils::global_to_pascal(utils::unbracket(&props[i + 1]));
+                                    let temp = match temp.as_str() {
+                                        "OflatBox" => "OFlatBox",
+                                        "OshadowBox" => "OShadowBox",
+                                        "RflatBox" => "RFlatBox",
+                                        "RshadowBox" => "RShadowBox",
+                                        _ => temp.as_str(),
+                                    };
+                                    imp += &format!(
+                                        "\t{}.set_down_frame(FrameType::{});\n",
+                                        &elem.ident, temp
+                                    );
+                                }
+                            } else {
+                                let temp = utils::global_to_pascal(utils::unbracket(&props[i + 1]));
+                                let temp = match temp.as_str() {
+                                    "OflatBox" => "OFlatBox",
+                                    "OshadowBox" => "OShadowBox",
+                                    "RflatBox" => "RFlatBox",
+                                    "RshadowBox" => "RShadowBox",
+                                    _ => temp.as_str(),
+                                };
                                 imp += &format!(
                                     "\t{}.set_down_frame(FrameType::{});\n",
-                                    &elem.ident,
-                                    utils::global_to_pascal(utils::unbracket(&props[i + 1]))
+                                    &elem.ident, temp
                                 );
                             }
                         }
                         "when" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!(
+                                        "\t{}.set_trigger(unsafe {{std::mem::transmute({})}});\n",
+                                        &elem.ident,
+                                        utils::unbracket(&props[i + 1])
+                                    );
+                                }
+                            } else {
                                 imp += &format!(
                                     "\t{}.set_trigger(unsafe {{std::mem::transmute({})}});\n",
                                     &elem.ident,
@@ -289,7 +414,19 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "tooltip" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!(
+                                        "\t{}.set_tooltip({});\n",
+                                        &elem.ident,
+                                        if unsafe { crate::parser::PROGRAM.i18n } {
+                                            format!("&tr!(\"{}\")", utils::unbracket(&props[i + 1]))
+                                        } else {
+                                            format!("\"{}\"", utils::unbracket(&props[i + 1]))
+                                        }
+                                    );
+                                }
+                            } else {
                                 imp += &format!(
                                     "\t{}.set_tooltip({});\n",
                                     &elem.ident,
@@ -302,7 +439,15 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "maximum" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!(
+                                        "\t{}.set_maximum({} as _);\n",
+                                        &elem.ident,
+                                        utils::unbracket(&props[i + 1])
+                                    );
+                                }
+                            } else {
                                 imp += &format!(
                                     "\t{}.set_maximum({} as _);\n",
                                     &elem.ident,
@@ -311,7 +456,15 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "minimum" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!(
+                                        "\t{}.set_minimum({} as _);\n",
+                                        &elem.ident,
+                                        utils::unbracket(&props[i + 1])
+                                    );
+                                }
+                            } else {
                                 imp += &format!(
                                     "\t{}.set_minimum({} as _);\n",
                                     &elem.ident,
@@ -320,7 +473,15 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "step" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!(
+                                        "\t{}.set_step({} as _, 1);\n",
+                                        &elem.ident,
+                                        utils::unbracket(&props[i + 1])
+                                    );
+                                }
+                            } else {
                                 imp += &format!(
                                     "\t{}.set_step({} as _, 1);\n",
                                     &elem.ident,
@@ -329,7 +490,31 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "value" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    let val = if t.contains("Button") {
+                                        let b = utils::unbracket(&props[i + 1])
+                                            .parse::<i32>()
+                                            .expect("Buttons should have integral values");
+                                        if b != 0 {
+                                            "true".to_string()
+                                        } else {
+                                            "false".to_string()
+                                        }
+                                    } else if (t.contains("Input") || t.contains("Output"))
+                                        && !t.contains("Value")
+                                    {
+                                        if unsafe { crate::parser::PROGRAM.i18n } {
+                                            format!("&tr!(\"{}\")", utils::unbracket(&props[i + 1]))
+                                        } else {
+                                            format!("\"{}\"", utils::unbracket(&props[i + 1]))
+                                        }
+                                    } else {
+                                        format!("{} as _", utils::unbracket(&props[i + 1]))
+                                    };
+                                    imp += &format!("\t{}.set_value({});\n", &elem.ident, val);
+                                }
+                            } else {
                                 let val = if t.contains("Button") {
                                     let b = utils::unbracket(&props[i + 1])
                                         .parse::<i32>()
@@ -354,13 +539,17 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "type" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
-                                if props[i + 1] != "Double" && t != "MenuItem" && t != "Submenu" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label"
+                                    && props[i + 1] != "Double"
+                                    && t != "MenuItem"
+                                    && t != "Submenu"
+                                {
                                     if t != "Output" {
                                         imp += &format!(
                                             "\t{}.set_type({}Type::{});\n",
                                             &elem.ident,
-                                            utils::fix_type(&t),
+                                            utils::fix_type(t),
                                             utils::global_to_pascal(utils::unbracket(
                                                 &props[i + 1]
                                             ))
@@ -372,10 +561,33 @@ pub fn generate(ast: &[parser::Token]) -> String {
                                         );
                                     }
                                 }
+                            } else if props[i + 1] != "Double" && t != "MenuItem" && t != "Submenu"
+                            {
+                                if t != "Output" {
+                                    imp += &format!(
+                                        "\t{}.set_type({}Type::{});\n",
+                                        &elem.ident,
+                                        utils::fix_type(t),
+                                        utils::global_to_pascal(utils::unbracket(&props[i + 1]))
+                                    );
+                                } else {
+                                    imp += &format!(
+                                        "\t{}.set_type(InputType::from_i32(12));\n",
+                                        &elem.ident,
+                                    );
+                                }
                             }
                         }
                         "align" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!(
+                                        "\t{}.set_align(unsafe {{std::mem::transmute({})}});\n",
+                                        &elem.ident,
+                                        utils::unbracket(&props[i + 1])
+                                    );
+                                }
+                            } else {
                                 imp += &format!(
                                     "\t{}.set_align(unsafe {{std::mem::transmute({})}});\n",
                                     &elem.ident,
@@ -384,18 +596,32 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "shortcut" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
-                                if t != "MenuItem" && t != "Submenu" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" && t != "MenuItem" && t != "Submenu" {
                                     imp += &format!(
                                         "\t{}.set_shortcut(unsafe {{std::mem::transmute({})}});\n",
                                         &elem.ident,
                                         utils::unbracket(&props[i + 1])
                                     );
                                 }
+                            } else if t != "MenuItem" && t != "Submenu" {
+                                imp += &format!(
+                                    "\t{}.set_shortcut(unsafe {{std::mem::transmute({})}});\n",
+                                    &elem.ident,
+                                    utils::unbracket(&props[i + 1])
+                                );
                             }
                         }
                         "image" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!(
+                                    "\t{0}.set_image(Some(SharedImage::load(\"{1}\").expect(\"Could not find image: {1}\")));\n",
+                                    &elem.ident,
+                                    utils::unbracket(&props[i + 1]),
+                                );
+                                }
+                            } else {
                                 imp += &format!(
                                     "\t{0}.set_image(Some(SharedImage::load(\"{1}\").expect(\"Could not find image: {1}\")));\n",
                                     &elem.ident,
@@ -404,24 +630,42 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "hide" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!("\t{}.hide();\n", &elem.ident,);
+                                }
+                            } else {
                                 imp += &format!("\t{}.hide();\n", &elem.ident,);
                             }
                         }
                         "modal" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!("\t{}.make_modal(true);\n", &elem.ident,);
+                                }
+                            } else {
                                 imp += &format!("\t{}.make_modal(true);\n", &elem.ident,);
                             }
                         }
                         "resizable" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
-                                if is_parent {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" && is_parent {
                                     imp += &format!("\t{}.make_resizable(true);\n", &elem.ident,);
                                 }
+                            } else if is_parent {
+                                imp += &format!("\t{}.make_resizable(true);\n", &elem.ident,);
                             }
                         }
                         "size_range" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!(
+                                        "\t{}.size_range({});\n",
+                                        &elem.ident,
+                                        utils::unbracket(&props[i + 1].replace(" ", ", "))
+                                    );
+                                }
+                            } else {
                                 imp += &format!(
                                     "\t{}.size_range({});\n",
                                     &elem.ident,
@@ -430,12 +674,12 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             }
                         }
                         "callback" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
-                                if t != "MenuItem" && t != "Submenu" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" && t != "MenuItem" && t != "Submenu" {
                                     let cb = utils::unbracket(&props[i + 1]);
-                                    if cb.starts_with("{")
+                                    if cb.starts_with('{')
                                         || cb.starts_with("move")
-                                        || cb.starts_with("|")
+                                        || cb.starts_with('|')
                                     {
                                         imp +=
                                             &format!("\t{}.set_callback({});\n", &elem.ident, cb);
@@ -446,10 +690,27 @@ pub fn generate(ast: &[parser::Token]) -> String {
                                         );
                                     };
                                 }
+                            } else if t != "MenuItem" && t != "Submenu" {
+                                let cb = utils::unbracket(&props[i + 1]);
+                                if cb.starts_with('{')
+                                    || cb.starts_with("move")
+                                    || cb.starts_with('|')
+                                {
+                                    imp += &format!("\t{}.set_callback({});\n", &elem.ident, cb);
+                                } else {
+                                    imp += &format!(
+                                        "\t{}.set_callback(move |{}| {{ \n\t    {} \n\t}});\n",
+                                        &elem.ident, &elem.ident, cb
+                                    );
+                                };
                             }
                         }
                         "code0" | "code1" | "code2" | "code3" => {
-                            if utils::unbracket(&props[i - 1]) != "label" {
+                            if let Some(p) = props.get(i.wrapping_sub(1)) {
+                                if p != "label" {
+                                    imp += &format!("\t{}\n", utils::unbracket(&props[i + 1]));
+                                }
+                            } else {
                                 imp += &format!("\t{}\n", utils::unbracket(&props[i + 1]));
                             }
                         }
@@ -526,11 +787,11 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             },
                             if let Some(cb) = cb {
                                 let cb = utils::unbracket(&cb);
-                                if cb.starts_with("{")
+                                if cb.starts_with('{')
                                     || cb.starts_with("move")
-                                    || cb.starts_with("|")
+                                    || cb.starts_with('|')
                                 {
-                                    format!("{}", cb)
+                                    cb.to_string()
                                 } else {
                                     format!("move |{}| {{ \n\t\t{} \n\t}}", &parent, cb)
                                 }
@@ -555,13 +816,11 @@ pub fn generate(ast: &[parser::Token]) -> String {
                             ctor.clear();
                         }
                         if let parser::TokenType::Scope(false, _) = last_ast {
-                            if let Some(last_scope) = last_scope {
-                                if let parser::TokenType::Scope(false, last_parent) = last_scope {
-                                    if let Some(l) = last_parent.last() {
-                                        if l.contains("Submenu") || l.contains("Fl_") {
-                                            subs.pop();
-                                            gparent.pop();
-                                        }
+                            if let Some(parser::TokenType::Scope(false, last_parent)) = last_scope {
+                                if let Some(l) = last_parent.last() {
+                                    if l.contains("Submenu") || l.contains("Fl_") {
+                                        subs.pop();
+                                        gparent.pop();
                                     }
                                 }
                             }
